@@ -22,5 +22,5 @@ NULL
 
 
 .onAttach <- function(libname, pkgname) {
-      packageStartupMessage("BacArena paper: https://doi.org/10.1371/journal.pcbi.1005544\n Tutorial: https://CRAN.R-project.org/package=BacArena/vignettes/BacArena-Introduction.pdf\n Development and help: https://github.com/euba/bacarena")
+      packageStartupMessage("BacArena paper: https://doi.org/10.1371/journal.pcbi.1005544\n Tutorials: https://bacarena.github.io\n Model import from SBML: https://github.com/euba/BacArena/wiki/Model-import\n Development and help: https://github.com/euba/bacarena")
 }
